# -*- coding: utf-8 -*-
# @Time    : 2021/8/7 18:15
# @Author  : liumingming
# @FileName: aiqi_bluetooth_uuid.py
# @Company ：http://www.iqi-inc.com/



class aiqi_bluetooth_uuid_class:
    Aiqi_Bluetooth_Services_Uuid = '6e400001-b5a3-f393-e0a9-e50e24dcca9e'
    Aiqi_Bluetooth_Rx_Char_Uuid = '6e400002-b5a3-f393-e0a9-e50e24dcca9e'
    Aiqi_Bluetooth_Tx_Char_Uuid = '6e400003-b5a3-f393-e0a9-e50e24dcca9e'